@@ -1,7 +1,7 @@
 /** \file  queryparser.h
  *  \brief parsing a user query string to build a Xapian::Query object
  */
-/* Copyright (C) 2005,2006,2007,2008,2009 Olly Betts
+/* Copyright (C) 2005,2006,2007,2008,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +32,7 @@
 
 namespace Xapian {
 
+class Database;
 class Stem;
 
 /// Base class for stop-word decision functor.
@@ -49,7 +50,6 @@ class XAPIAN_VISIBILITY_DEFAULT Stopper {
 
 /// Simple implementation of Stopper class - this will suit most users.
 class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
-  private:
     std::set<std::string> stop_words;
 
   public:
@@ -57,11 +57,12 @@ class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
     SimpleStopper() { }
 
     /// Initialise from a pair of iterators.
-#ifndef __SUNPRO_CC
+#if ! defined __SUNPRO_CC || __SUNPRO_CC - 0 >= 0x580
     template <class Iterator>
     SimpleStopper(Iterator begin, Iterator end) : stop_words(begin, end) { }
 #else
-    // Sun's C++ doesn't cope with the Iterator pointing to const char *.
+    // Older versions of Sun's C++ compiler don't cope with the Iterator
+    // pointing to const char *.
     template <class Iterator>
     SimpleStopper(Iterator begin, Iterator end) {
 	while (begin != end) stop_words.insert(*begin++);
@@ -76,9 +77,6 @@ class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
 	return stop_words.find(term) != stop_words.end();
     }
 
-    /// Destructor.
-    virtual ~SimpleStopper() { }
-
     /// Return a string describing this object.
     virtual std::string get_description() const;
 };
@@ -91,7 +89,7 @@ struct XAPIAN_VISIBILITY_DEFAULT ValueRangeProcessor {
     /** Check for a valid range of this type.
      *
      *  If this ValueRangeProcessor recognises BEGIN..END it returns the
-     *  value number of range filter on.  Otherwise it returns
+     *  value number to range filter on.  Otherwise it returns
      *  Xapian::BAD_VALUENO.
      */
     virtual Xapian::valueno operator()(std::string &begin, std::string &end) = 0;
@@ -102,28 +100,49 @@ struct XAPIAN_VISIBILITY_DEFAULT ValueRangeProcessor {
  *  The end points can be any strings.
  */
 class XAPIAN_VISIBILITY_DEFAULT StringValueRangeProcessor : public ValueRangeProcessor {
+  protected:
     Xapian::valueno valno;
 
+  private:
+    bool prefix;
+    std::string str;
+
   public:
     /** Constructor.
      *
      *  @param valno_	The value number to return from operator().
      */
     StringValueRangeProcessor(Xapian::valueno valno_)
-	: valno(valno_) { }
+	: valno(valno_), str() { }
 
-    /// Any strings are valid as begin and end.
-    Xapian::valueno operator()(std::string &, std::string &) {
-	return valno;
-    }
+    /** Constructor.
+     *
+     *  @param valno_	The value number to return from operator().
+     *  @param str_     A string to look for to recognise values as belonging
+     *                  to this range.
+     *  @param prefix_	Flag specifying whether to check for str_ as a prefix
+     *			or a suffix.
+     */
+    StringValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
+			      bool prefix_ = true)
+	: valno(valno_), prefix(prefix_), str(str_) { }
+
+    /** Check for a valid range of this type.
+     *
+     *  If no prefix or suffix is specified, then this always returns the
+     *  value slot specified at construction time.
+     *
+     *  If a prefix or suffix is specified, this is checked for first, and
+     *  it it doesn't match, this method returns Xapian::BAD_VALUENO.
+     */
+    Xapian::valueno operator()(std::string &, std::string &);
 };
 
 /** Handle a date range.
  *
  *  Begin and end must be dates in a recognised format.
  */
-class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public ValueRangeProcessor {
-    Xapian::valueno valno;
+class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public StringValueRangeProcessor {
     bool prefer_mdy;
     int epoch_year;
 
@@ -140,7 +159,51 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public ValueRangeProce
      */
     DateValueRangeProcessor(Xapian::valueno valno_, bool prefer_mdy_ = false,
 			    int epoch_year_ = 1970)
-	: valno(valno_), prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
+	: StringValueRangeProcessor(valno_),
+	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
+
+    /** Constructor.
+     *
+     *  @param valno_	    The value number to return from operator().
+     *
+     *  @param str_     A string to look for to recognise values as belonging
+     *                  to this date range.
+     *
+     *  @param prefix_  Whether to look for the string at the start or end of
+     *                  the values.  If true, the string is a prefix; if
+     *                  false, the string is a suffix (default: true).
+     *
+     *  @param prefer_mdy_  Should ambiguous dates be interpreted as
+     *			    month/day/year rather than day/month/year?
+     *			    (default: false)
+     *
+     *  @param epoch_year_  Year to use as the epoch for dates with 2 digit
+     *			    years (default: 1970, so 1/1/69 is 2069 while
+     *			    1/1/70 is 1970).
+     *
+     *  The string supplied in str_ is used by @a operator() to decide whether
+     *  the pair of strings supplied to it constitute a valid range.  If
+     *  prefix_ is true, the first value in a range must begin with str_ (and
+     *  the second value may optionally begin with str_);
+     *  if prefix_ is false, the second value in a range must end with str_
+     *  (and the first value may optionally end with str_).
+     *
+     *  If str_ is empty, the setting of prefix_ is irrelevant, and no special
+     *  strings are required at the start or end of the strings defining the
+     *  range.
+     *
+     *  The remainder of both strings defining the endpoints must be valid
+     *  dates.
+     *
+     *  For example, if str_ is "created:" and prefix_ is true, and the range
+     *  processor has been added to the queryparser, the queryparser will
+     *  accept "created:1/1/2000..31/12/2001".
+     */
+    DateValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
+			    bool prefix_ = true,
+			    bool prefer_mdy_ = false, int epoch_year_ = 1970)
+	: StringValueRangeProcessor(valno_, str_, prefix_),
+	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
 
     /** Check for a valid range of this type.
      *
@@ -151,21 +214,6 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public ValueRangeProce
     Xapian::valueno operator()(std::string &begin, std::string &end);
 };
 
-// Xapian 1.0.0 and 1.0.1 had a conceptually broken implementation of
-// NumberValueRangeProcessor which we quickly told people to avoid using.  But
-// to keep ABI compatibility, we should keep it around until the next
-// incompatible ABI change (in 1.1.0).  So we put the new
-// NumberValueRangeProcessor in a subnamespace and then pull it into this one
-// with "using v102::NumberValueRangeProcessor".  The old
-// NumberValueRangeProcessor still exists with default visibility, but isn't
-// declared in an external or internal header, so will only be used when
-// dynamically linking with application code built against Xapian 1.0.0 or
-// 1.0.1.
-#ifndef DOXYGEN
-// Hide the v102 namespace from Doxygen as it isn't user-visible.
-namespace v102 {
-#endif
-
 /** Handle a number range.
  *
  *  This class must be used on values which have been encoded using
@@ -173,18 +221,14 @@ namespace v102 {
  *  will sort in the same order as the numbers (the same values can be
  *  used to implement a numeric sort).
  */
-class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public ValueRangeProcessor {
-    Xapian::valueno valno;
-    bool prefix;
-    std::string str;
-
+class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public StringValueRangeProcessor {
   public:
     /** Constructor.
      *
      *  @param valno_   The value number to return from operator().
      */
     NumberValueRangeProcessor(Xapian::valueno valno_)
-	: valno(valno_), prefix(false) { }
+	: StringValueRangeProcessor(valno_) { }
 
     /** Constructor.
      *
@@ -220,7 +264,7 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public ValueRangePro
      */
     NumberValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
 			      bool prefix_ = true)
-	: valno(valno_), prefix(prefix_), str(str_) { }
+	: StringValueRangeProcessor(valno_, str_, prefix_) { }
 
     /** Check for a valid range of this type.
      *
@@ -234,14 +278,6 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public ValueRangePro
     Xapian::valueno operator()(std::string &begin, std::string &end);
 };
 
-#ifndef DOXYGEN
-}
-
-# ifndef XAPIAN_NO_V102_NUMBER_VRP
-using v102::NumberValueRangeProcessor;
-# endif
-#endif
-
 /// Build a Xapian::Query object from a user query string.
 class XAPIAN_VISIBILITY_DEFAULT QueryParser {
   public:
@@ -261,6 +297,10 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	/// Support AND, OR, etc even if they aren't in ALLCAPS.
 	FLAG_BOOLEAN_ANY_CASE = 8,
 	/** Support right truncation (e.g. Xap*).
+	 *
+	 *  Currently you can't use wildcards with boolean filter prefixes,
+	 *  or in a phrase (either an explicitly quoted one, or one implicitly
+	 *  generated by hyphens or other punctuation).
 	 *
 	 *  NB: You need to tell the QueryParser object which database to
 	 *  expand wildcards from by calling set_database.
@@ -281,6 +321,12 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 *  whitespace, to produce more stable results from interactive
 	 *  searches.
 	 *
+	 *  Currently FLAG_PARTIAL doesn't do anything if the final word
+	 *  in the query has a boolean filter prefix, or if it is in a phrase
+	 *  (either an explicitly quoted one, or one implicitly generated by
+	 *  hyphens or other punctuation).  It also doesn't do anything if
+	 *  if the final word is part of a value range.
+	 *
 	 *  NB: You need to tell the QueryParser object which database to
 	 *  expand wildcards from by calling set_database.
 	 */
@@ -391,7 +437,13 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     /** Get the current default operator. */
     Query::op get_default_op() const;
 
-    /// Specify the database being searched.
+    /** Specify the database being searched.
+     *
+     *  The database is used for wildcard expansion (FLAG_WILDCARD and
+     *  FLAG_PARTIAL), spelling correction (FLAG_SPELLING_CORRECTION), and
+     *  synonyms (FLAG_SYNONYM, FLAG_AUTO_SYNONYMS, and
+     *  FLAG_AUTO_MULTIWORD_SYNONYMS).
+     */
     void set_database(const Database &db);
 
     /** Parse a query.
@@ -402,10 +454,24 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *		multiple values with bitwise-or (|) (default FLAG_DEFAULT).
      *	@param default_prefix  The default term prefix to use (default none).
      *		For example, you can pass "A" when parsing an "Author" field.
+     *
+     *  @exception If the query string can't be parsed, then
+     *		   Xapian::QueryParserError is thrown.  You can get an English
+     *		   error message to report to the user by catching it and
+     *		   calling get_msg() on the caught exception.  The current
+     *		   possible values (in case you want to translate them) are:
+     *
+     *			@li Unknown range operation
+     *			@li parse error
+     *			@li Syntax: <expression> AND <expression>
+     *			@li Syntax: <expression> AND NOT <expression>
+     *			@li Syntax: <expression> NOT <expression>
+     *			@li Syntax: <expression> OR <expression>
+     *			@li Syntax: <expression> XOR <expression>
      */
     Query parse_query(const std::string &query_string,
-		      unsigned flags = FLAG_PHRASE|FLAG_BOOLEAN|FLAG_LOVEHATE,
-		      const std::string &default_prefix = "");
+		      unsigned flags = FLAG_DEFAULT,
+		      const std::string &default_prefix = std::string());
 
     /** Add a probabilistic term prefix.
      *
@@ -492,8 +558,19 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *
      *  @param field   The user visible field name
      *  @param prefix  The term prefix to map this to
+     *  @param exclusive If true, each document can have at most one value of
+     *			 the field, so Xapian should combine multiple values
+     *			 with OP_OR.  If false, each document can have multiple
+     *			 values of the field, so Xapian combine them with
+     *			 OP_AND, as we would with filters with different
+     *			 prefixes. [default: true]
      */
-    void add_boolean_prefix(const std::string & field, const std::string &prefix);
+    void add_boolean_prefix(const std::string &field, const std::string &prefix,
+			    bool exclusive);
+
+    /* FIXME:1.1.3: Merge two versions into one with optional parameter
+     * "exclusive", default true. */
+    void add_boolean_prefix(const std::string &field, const std::string &prefix);
 
     /// Iterate over terms omitted from the query as stopwords.
     TermIterator stoplist_begin() const;
