@@ -1,7 +1,7 @@
 /** @file version.h
  * @brief Define preprocesor symbols for the library version
  */
-// Copyright (C) 2002,2004,2005,2006,2007,2009 Olly Betts
+// Copyright (C) 2002,2004,2005,2006,2007,2008,2009,2010 Olly Betts
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -21,6 +21,9 @@
 #define XAPIAN_INCLUDED_VERSION_H
 
 #ifdef __GNUC__
+#if __GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ == 0)
+#error Xapian no longer supports GCC < 3.1
+#else
 #if !defined(__GXX_ABI_VERSION) || __GXX_ABI_VERSION != 102
 #error The C++ ABI version of compiler you are using does not match
 #error that of the compiler used to build the library. The versions
@@ -28,10 +31,11 @@
 #error The Xapian library was built with g++ 3.3.5
 #endif
 
+#endif
 #endif
 
 /// The version of Xapian as a C string literal.
-#define XAPIAN_VERSION "1.0.17"
+#define XAPIAN_VERSION "1.2.4"
 
 /** The major component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 1
@@ -41,19 +45,22 @@
 /** The minor component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 0
  */
-#define XAPIAN_MINOR_VERSION 0
+#define XAPIAN_MINOR_VERSION 2
 
 /** The revision component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 14
  */
-#define XAPIAN_REVISION 17
+#define XAPIAN_REVISION 4
+
+/// XAPIAN_HAS_BRASS_BACKEND Defined if the brass backend is enabled.
+#define XAPIAN_HAS_BRASS_BACKEND 1
+
+/// XAPIAN_HAS_CHERT_BACKEND Defined if the chert backend is enabled.
+#define XAPIAN_HAS_CHERT_BACKEND 1
 
 /// XAPIAN_HAS_FLINT_BACKEND Defined if the flint backend is enabled.
 #define XAPIAN_HAS_FLINT_BACKEND 1
 
-/// XAPIAN_HAS_QUARTZ_BACKEND Defined if the quartz backend is enabled.
-#define XAPIAN_HAS_QUARTZ_BACKEND 1
-
 /// XAPIAN_HAS_INMEMORY_BACKEND Defined if the inmemory backend is enabled.
 #define XAPIAN_HAS_INMEMORY_BACKEND 1
 
